from .exceptions import *
import os

class Validation:
    def __init__(self,**kwargs):
        self.arguments = kwargs
        self.column_type_mapping = {"file_size": int, "column_description" : dict, "font_size" : int, "save_file_path": str}
        self.allowed_data_types = ["int","str","boolean","datetime","float"]

    def validate_args(self):
        for key, value in self.arguments.items():
            if not self.column_type_mapping[key] == type(value):
                raise InvalidInputDataTypeException(f"The value of '{key}' argument should be of {self.column_type_mapping[key]} data type")

    def validate_file_path(self):
            if not os.path.exists(self.arguments["save_file_path"]): 
                raise InvalidPathException(f"Provided file path '{self.arguments['save_file_path']}' for saving output file does not exist")

    def validate_file_size_limit(self):
        if not 0 <= self.arguments["file_size"] <= 2000000000: 
            raise InvalidSizeException(f"The value of 'file_size' argument should be between 0 and 2GB")

    def validate_column_description(self):
        for key,value in self.arguments["column_description"].items():
            if type(key) == str and type(value) == str:
                if not value in self.allowed_data_types:
                    raise InvalidColumnDescriptionValueException(f"The values of 'column_description' dictionary should belong to the set of allowed data types i.e {self.allowed_data_types}")
            else:
                raise InvalidInputDataTypeException(f"The keys and values of 'column_description' dictionary should have key and value of string type")
            
    def validate_font_attributes(self):
        if not 1 <= self.arguments["font_size"] <= 1683:
            raise InvalidSizeException(f"The value of 'font_size' argument should be between 1 and 1683")


from .validator import Validation
from .helper_functions import *
import json

class JsonGenerator:
    def __init__(self, **kwargs):
        self.arguments = kwargs
        self.new_file_path = get_path_to_create_new_file("json")
        self.data_type_value = {"str": "generate_random_string", "int": "generate_random_int", "datetime": "generate_current_datetime", "float": "generate_random_float", "boolean": "generate_random_boolean"}

    def prepare_row(self):
        return {k : globals()[self.data_type_value[v]]() for k,v in self.arguments["column_description"].items()}

    def generate_file(self):
        data = []
        current_file_size = 0
        while(current_file_size<=self.arguments["file_size"]):
            with open(self.new_file_path, 'w') as json_file:  
                data.append(self.prepare_row())
                json.dump(data, json_file, default=str, indent=4)
                current_file_size = json_file.tell() 

    def main(self):
        obj = Validation(**self.arguments)
        obj.validate_args()
        if self.arguments["save_file_path"]:
            obj.validate_file_path()
            self.new_file_path = get_path_to_create_new_file("json", user_input_path = self.arguments["save_file_path"])
        else:
            self.new_file_path = get_path_to_create_new_file("json")
        obj.validate_file_size_limit()
        obj.validate_column_description()
        print(f"Generating the file at {self.new_file_path}")
        self.generate_file()
        print(f"Generated the file at {self.new_file_path}")


from .validator import Validation
from .helper_functions import *

class TxtGenerator:
    def __init__(self, **kwargs):
        self.arguments = kwargs
        self.new_file_path = get_path_to_create_new_file("txt")

    def generate_file(self):
        with open(self.new_file_path, 'a') as txt_file:
            current_file_size = 0
            while(current_file_size<=self.arguments["file_size"]): 
                txt = generate_random_string()
                txt_file.write(txt+ '\n')
                current_file_size = txt_file.tell() 

    def main(self):
        obj = Validation(**self.arguments)
        obj.validate_args()
        if self.arguments["save_file_path"]:
            obj.validate_file_path()
            self.new_file_path = get_path_to_create_new_file("txt", user_input_path = self.arguments["save_file_path"])
        else:
            self.new_file_path = get_path_to_create_new_file("txt")
        obj.validate_file_size_limit()
        print(f"Generating the file at {self.new_file_path}")
        self.generate_file()
        print(f"Generated the file at {self.new_file_path}")



from .validator import Validation
from .helper_functions import *
import csv

class CsvGenerator:
    def __init__(self, **kwargs):
        self.arguments = kwargs
        self.new_file_path = ""
        self.data_type_value = {"str": "generate_random_string", "int": "generate_random_int", "datetime": "generate_current_datetime", "float": "generate_random_float", "boolean": "generate_random_boolean"}
        self.column_headers = []
        self.column_data_type = []

    def prepare_column(self):
        for k,v in self.arguments["column_description"].items():
            self.column_headers.append(k)
            self.column_data_type.append(v)

    def prepare_row(self):
        return [globals()[self.data_type_value[i]]() for i in self.column_data_type]
    
    def generate_file(self):
        with open(self.new_file_path, 'w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file)
            current_file_size = 0
            csv_writer.writerow(self.column_headers)
            while(current_file_size<=self.arguments["file_size"]): 
                csv_writer.writerow(self.prepare_row())
                current_file_size = csv_file.tell() 
    
    def main(self):
        obj = Validation(**self.arguments)
        obj.validate_args()
        if self.arguments["save_file_path"]:
            obj.validate_file_path()
            self.new_file_path = get_path_to_create_new_file("csv", user_input_path = self.arguments["save_file_path"])
        else:
            self.new_file_path = get_path_to_create_new_file("csv")
        obj.validate_file_size_limit()
        obj.validate_column_description()
        self.prepare_column()
        print(f"Generating the file at {self.new_file_path}")
        self.generate_file()
        print(f"Generated the file at {self.new_file_path}")



from .validator import Validation
from .helper_functions import *
from docx import Document
from docx.shared import Pt
import os

class WordGenerator:
    def __init__(self, **kwargs):
        self.arguments = kwargs
        self.new_file_path = get_path_to_create_new_file("docx")

    def generate_file(self):
        doc = Document()
        current_file_size = 0
        while(current_file_size<=self.arguments["file_size"]): 
            word_line = generate_random_string()
            paragraph = doc.add_paragraph(word_line)
            font = paragraph.runs[0].font
            font.size = Pt(self.arguments["font_size"])
            doc.save(self.new_file_path)
            current_file_size = os.path.getsize(self.new_file_path)
    
    def main(self):
        obj = Validation(**self.arguments)
        obj.validate_args()
        if self.arguments["save_file_path"]:
            obj.validate_file_path()
            self.new_file_path = get_path_to_create_new_file("docx", user_input_path = self.arguments["save_file_path"])
        else:
            self.new_file_path = get_path_to_create_new_file("docx")
        obj.validate_file_size_limit()
        obj.validate_font_attributes()
        print(f"Generating the file at {self.new_file_path}")
        self.generate_file()
        print(f"Generated the file at {self.new_file_path}")



from datetime import datetime
import random 
import os

def get_path_to_create_new_file(file_type, user_input_path = ""):
    try:
        if user_input_path:
            if user_input_path.endswith('/'):
                path = f'{user_input_path}Dummy.{file_type}'
            else:
                path = f'{user_input_path}/Dummy.{file_type}'
        else:
            path = f'{os.path.expanduser("~")}/Downloads/Dummy.{file_type}'
    except Exception as e:
        return e
    return path

def zero_byte_file(file_type):
    with open(f'{get_path_to_create_new_file(file_type)}',"w"):
        return None

def generate_random_string():
    string = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
    return ''.join(random.sample(string,len(string)))

def generate_random_int():
    return random.getrandbits(32)

def generate_random_float():
    return random.random()

def generate_random_boolean():
    return random.choice([True, False])

def generate_current_datetime():
    return datetime.now()


class InvalidColumnDescriptionValueException(Exception):
    """Exception raised when an invalid value is encountered in a column description."""
    def __init__(self, message : str):
        self.message = message
        super().__init__(self.message)

class InvalidInputDataTypeException(Exception):
    """Exception raised when the input's data type is invalid."""
    def __init__(self, message : str):
        self.message = message
        super().__init__(self.message)

class InvalidSizeException(Exception):
    """Exception raised when an invalid size is encountered."""
    def __init__(self, message: str):
        self.message = message
        super().__init__(self.message)

class InvalidPathException(Exception):
    """Exception raised when an invalid save file path is provided."""
    def __init__(self, message: str):
        self.message = message
        super().__init__(self.message)


from .csv_generator import CsvGenerator
from .excel_generator import ExcelGenerator
from .json_generator import JsonGenerator
from .txt_generator import TxtGenerator
from .word_generator import WordGenerator

"""
File size argument should be in bytes
Column description, save file path and font size arguments are optional
"""

def generate_csv(file_size : int, column_description : dict = {"S.No" : "int", "Product_Name" : "str", "Product_Status": "boolean", "Delivery_Date": "datetime", "Rate": "float"}, save_file_path : str = ""):
    csv_obj = CsvGenerator(file_size=file_size, column_description=column_description, save_file_path = save_file_path)
    csv_obj.main()

def generate_json(file_size : int, column_description : dict = {"S.No" : "int", "Product_Name" : "str", "Product_Status": "boolean", "Delivery_Date": "datetime", "Rate": "float"}, save_file_path : str = ""):
    json_obj = JsonGenerator(file_size=file_size, column_description=column_description, save_file_path = save_file_path)
    json_obj.main()

def generate_excel(file_size : int, column_description : dict = {"S.No" : "int", "Product_Name" : "str", "Product_Status": "boolean", "Delivery_Date": "datetime", "Rate": "float"}, save_file_path : str = ""):
    excel_obj = ExcelGenerator(file_size=file_size, column_description=column_description, save_file_path = save_file_path)
    excel_obj.main()

def generate_txt(file_size : int, save_file_path : str = ""):
    txt_obj = TxtGenerator(file_size=file_size, save_file_path = save_file_path)
    txt_obj.main()

def generate_word(file_size : int, font_size : int = 12, save_file_path : str = ""):
    word_obj = WordGenerator(file_size=file_size, font_size=font_size, save_file_path = save_file_path)
    word_obj.main()



from setuptools import setup

VERSION = '1.1.4' 
DESCRIPTION = 'Generate different types (json, csv, excel, word, txt) of dummy files'

# Setting up
setup(
        name="generate-dummyf",
        version=VERSION,
        author="Rubina Karki",
        author_email="rubinakarki369@gmail.com",
        description=DESCRIPTION,
        long_description=open('README.md').read(),
        long_description_content_type="text/markdown",
        packages=["generate_dummyf"],
        url='https://github.com/rubinakarkii/generate-dummyf',
        license="LICENSE",
        install_requires=['openpyxl','python-docx'], 
)